// ===========================================================================
//	GraphGA_PaneIDs.h			1995 Timo Eloranta		All rights reserved.
// ===========================================================================
//
//	By convention, the WIND resource ID, the PPob resource ID, and the Pane ID
//  of the window object in the PPob are the same.

#pragma once

//	 TimGA creator and filetype

const OSType	Creator_GraphGA		= 'TiGA';
const OSType	FileType_GraphGA	= 'TEXT';

//	 TimGA definitions

#define	WIND_GraphGADoc				1000
#define WIND_NewAbout				1001

#define titl_GraphInfo				1100
#define	capt_Name					1103
#define	capt_Nodes					1105
#define	capt_Edges					1107
#define capt_GridSize				1109

#define titl_PopInfo				1200
#define capt_PopSize				1203
#define capt_CurGeneration			1205
#define capt_RunningTime			1207
#define capt_LeastCrossings			1209
#define capt_AvgCrossings			1211
#define capt_IterState				1213

#define pane_BestInfo				1300
#define capt_BestEverFitness		1303
#define capt_BestEverGeneration		1305
#define capt_NoChangeGenerations	1307
#define capt_NoChangeTime			1309

#define pane_GraphDrawing			1500

//	 Sleep times

#define	kSleepTimeFG				3L		// Foreground sleep time
#define	kSleepTimeBG				120L	// Background sleep time

#define	kBG_MaxSpendTime			12L

//	 TimGA commands

#define	cmd_CloseGraph				1100

#define	cmd_IterStart				1001
#define	cmd_IterPause				1002
#define	cmd_IterReinitialize 		1003
#define cmd_IterSuspend				1004
#define cmd_IterResume				1005

#define cmd_General					3000
#define cmd_Selection				2001
#define cmd_Recombination			2002
#define cmd_Termination				2003
#define cmd_Evaluation				4000
												// Drawing MENU
#define cmd_ShowDrawing				1501
#define cmd_ShowGrid				1502
#define cmd_ShowEdges				1503
#define cmd_ShowNodes				1504
#define cmd_BrightenGrid			1505
#define cmd_BlackAndWhite			1506
#define cmd_IncreaseSquareSize		1507
#define cmd_DecreaseSquareSize		1508
#define cmd_SetSquareSize			1509

#define cmd_SetGeneral				-3000
#define cmd_SetProbabilities		-3001
#define cmd_SetSelection			-3002
#define cmd_SetTermination			-3003
#define cmd_SetEvaluation			-4000

//	 Alerts

#define ALRT_InvalidSquareSize		129
#define ALRT_Offscreen				130
#define ALRT_Memory					131
#define ALRT_TooManyEdges			132
#define ALRT_TooFewNodes			133
#define ALRT_TooManyNodes			134
#define ALRT_TooSmallGrid			135
#define ALRT_DivideByZero			136

//	 Strings

#define	STRx_Untitled				300
#define STRx_Menus					400
#define str_ShowDrawing				1
#define str_HideDrawing				2
#define str_ShowGrid				3
#define str_HideGrid				4
#define str_ShowEdges				5
#define str_HideEdges				6
#define str_ShowNodes				7
#define str_HideNodes				8
#define str_Start					9
#define str_Continue				10
#define str_BrightenGrid			11
#define str_DarkenGrid				12
#define str_ToBlackAndWhite			13
#define str_SwitchToColor			14
#define STRx_IterStates				500
#define str_ReadyToStart			1
#define str_Running					2
#define str_Paused					3
#define str_Finished				4
#define str_Walking					5
#define str_NoGraph					6

//	 Sounds

#define snd_Done					300

//	 Cursors

#define curs_First					256
#define curs_Qty					4


//	 New Graph Dialog

#define	WIND_NewGraph				1100
#define edit_Nodes					3001
#define edit_Edges					3002

//	 Set Square Size Dlg

#define WIND_SetSize				1002
#define edit_SquareSize				1001

//	 Common Dialog Stuff

#define but_FactorySettings			3097
#define msg_FactorySettings			5000
#define pict_Base					200			
#define pict_Slider					201	
#define pict_Selected				202	

//	 General Dialog

#define	WIND_General				3000
#define slid_Pop					2001
#define slid_Grid					2002
#define capt_Pop					2011
#define capt_Grid					2022
#define edit_Ticks					2031

//	 Probabilities Dialog

#define	WIND_Prob					3001
#define slid_CrossSlider			2001
#define slid_MutaSlider				2002
#define msg_CrossSlider				2001
#define msg_MutaSlider				2002
#define capt_Crossover				2011
#define capt_Mutation				2022

//	 Selection Dialog

#define	WIND_Select					3002
#define slid_StepSlider				2001
#define slid_MinSlider				2002
#define msg_StepSlider				2001
#define msg_MinSlider				2002
#define capt_Step					2011
#define capt_Minimum				2022
#define cbox_Elitism				2035
#define msg_Elitism					2035
#define rbut_Automatic				2051
#define msg_Automatic				2051
#define rbut_Manual					2052
#define msg_ManualStep				2052

//	 Termination Dialog

#define	WIND_Terminate				3003
#define cbox_MaxGen					2011
#define msg_MaxGen					2011
#define edit_MaxGen					2013
#define cbox_MaxGenNo				2012
#define msg_MaxGenNo				2012
#define edit_MaxGenNo				2014
#define cbox_MaxTime				2021
#define msg_MaxTime					2021
#define edit_MaxTime				2023
#define cbox_MaxTimeNo				2022
#define msg_MaxTimeNo				2022
#define edit_MaxTimeNo				2024
#define cbox_NoCrossings			2031
#define msg_NoCrossings				2031

//	 Evaluation Dialog

#define	WIND_Evaluate				4000
#define cbox_CrossingsRule			4001
#define msg_CrossingsRule			4001
#define edit_1						4011
#define edit_2						4012
#define edit_3						4013
#define edit_4						4014
#define edit_5						4015
#define edit_6						4016
#define edit_7						4017

//   Defaults

#define	DEFAULT_GRID_SIZE			40
#define	DEFAULT_POP_SIZE			8
#define	DEFAULT_SPEND_TIME			20

#define DEFAULT_CROSSINGS_RULE		1
#define DEFAULT_MULTIP_1			2
#define DEFAULT_MULTIP_2			2
#define DEFAULT_MULTIP_3			5
#define DEFAULT_MULTIP_4			2
#define DEFAULT_MULTIP_5			4
#define DEFAULT_MULTIP_6			1
#define DEFAULT_MULTIP_7			1

#define	DEFAULT_NEW_GRAPH_NODES		10L
#define	DEFAULT_NEW_GRAPH_EDGES		15L

#define	DEFAULT_CROSSOVER_PROB		5
#define	DEFAULT_MUTATION_PROB		45

#define	DEFAULT_SELECTION_STEP		20
#define	DEFAULT_SELECTION_MIN		20
#define	DEFAULT_SELECTION_AUTOM 	1
#define	DEFAULT_ELITISM				1

#define	DEFAULT_MAX_GEN_TOTAL		1000L
#define	DEFAULT_MAX_GEN_NO_CHANGE 	100L
#define	DEFAULT_MAX_TIME_TOTAL		3600L	// ticks
#define	DEFAULT_MAX_TIME_NO_CHANGE	600L	// ticks
